import { Server } from 'socket.io'

import { TSocket } from '#application/types'
import { Character } from '#entities/character'
import SocketManager from '#managers/socketManager'
import ZoneManager from '#managers/zoneManager'
import TeleportService from '#services/teleportService'

class ZoneCharacter {
  public readonly character: Character
  public isMoving: boolean = false
  public currentPath: Array<{ x: number; y: number }> | null = null

  constructor(character: Character) {
    this.character = character
  }

  public async savePosition() {
    await this.character.setPositionX(this.character.positionX).setPositionY(this.character.positionY).setRotation(this.character.rotation).setZone(this.character.zone).update()
  }

  public async teleport(zoneId: number, targetX: number, targetY: number): Promise<void> {
    await TeleportService.teleportCharacter(this.character.id, {
      targetZoneId: zoneId,
      targetX,
      targetY
    })
  }

  public async disconnect(socket: TSocket, io: Server): Promise<void> {
    try {
      // Stop any movement and save final position
      this.isMoving = false
      this.currentPath = null
      await this.savePosition()

      // Leave zone and remove from manager
      if (this.character.zone) {
        socket.leave(this.character.zone.id)
        ZoneManager.removeCharacter(this.character.id)

        // Notify zone players
        io.in(this.character.zone.id).emit('zone:character:leave', this.character.id)
      }

      // Notify all players
      io.emit('character:disconnect', this.character.id)
    } catch (error) {
      console.error(`Error disconnecting character ${this.character.id}:`, error)
    }
  }
}

export default ZoneCharacter
